/********************************************************************
 *
 *	TITLE:	   header
 *
 *	DATE:	   05-26-82
 *
 *	ABSTRACT:  .
 *
 *	LANGUAGE DEPENDENCIES:
 *
 *		C86     VERSION 1.0
 *
 *********************************************************************/
#include <sys/types.h>
#include <sys/timeb.h>
#include <curses.h>
#include "common.def"

	/*
	 *  External variables
	 */

extern int end_io;	/*  Last crt line of io tests			*/
extern int end_8087;	/*  Last crt line of 8087 tests			*/
extern int h_update;	/*  Time in seconds of header update		*/
extern bool idleps;	/*  Signifies if idle header is displayed	*/


/********************************************************************
 *
 *	TITLE:	   header
 *
 *	CALLING SEQUENCE:
 *
 *		   header();
 *
 *	INTERFACE VARIABLES:
 *
 *		   .
 *              
 *	CALLS:	   .
 *
 *	ABSTRACT:  .
 *
 *
 *********************************************************************/
fheader()
{
	mvaddstr( 1,  1, "File name" );
	mvaddstr( 0, 22, "Buffer" );
	mvaddstr( 1, 23, "size" );
}

/********************************************************************
 *
 *	TITLE:	   gen_bars
 *
 *	CALLING SEQUENCE:
 *
 *		   gen_bars( b_line, e_line, io_test );
 *
 *	INTERFACE VARIABLES:
 *
 *		   .
 *              
 *	CALLS:	   .
 *
 *	ABSTRACT:  .
 *
 *
 *********************************************************************/
gen_bars( b_line, e_line, io_test )
int b_line;		/*  Begining line to generate bars		*/
int e_line;		/*  Ending line to generate bars		*/
bool io_test;		/*  Generate bars of io tests or 8087 tests	*/
{
int i;			/*  Typical loop counter			*/

	/*
	 *  Display bars 
	 */

	for ( i = b_line; i <= e_line; i++ ) {

		if ( io_test ) {

			/*
			 *  Display bars for io test
			 */

			mvaddch( i, 20, '|' );
			mvaddch( i, 29, '|' );
			mvaddch( i, 36, '|' );
			mvaddch( i, 44, '|' );
			mvaddch( i, 56, '|' );
			mvaddch( i, 64, '|' );
		}
		else {

			/*
			 *  Display bars for 8087 test
			 */

			mvaddch( i,  8, '|' );
			mvaddch( i, 16, '|' );
			mvaddch( i, 24, '|' );
			mvaddch( i, 32, '|' );
			mvaddch( i, 40, '|' );
			mvaddch( i, 48, '|' );
			mvaddch( i, 56, '|' );
		}

	}

}

/********************************************************************
 *
 *	TITLE:	   h_io
 *
 *	CALLING SEQUENCE:
 *
 *		   h_io();
 *
 *	INTERFACE VARIABLES:
 *
 *		   .
 *              
 *	CALLS:	   .
 *
 *	ABSTRACT:  .
 *
 *
 *********************************************************************/
h_io()
{
	fheader();
	mvaddstr( 1, 31, "Pass" );
	mvaddstr( 0, 38, "File" );
	mvaddstr( 1, 39, "op" );
	mvaddstr( 0, 48, "File" );
	mvaddstr( 1, 48, "Pointer" );
	mvaddstr( 1, 58, "Value" );
	mvaddstr( 1, 66, "Operation" );
}

/********************************************************************
 *
 *	TITLE:	   h_8087
 *
 *	CALLING SEQUENCE:
 *
 *		   h_8087();
 *
 *	INTERFACE VARIABLES:
 *
 *		   .
 *              
 *	CALLS:	   .
 *
 *	ABSTRACT:  .
 *
 *
 *********************************************************************/
h_8087()
{
int h_line;		/*  CRT line for 8087 header			*/

	/*
	 *  If there are no io tests then the 8087 header starts at line
	 *  one, otherwise the 8087 header starts at the end of the io
	 *  tests plus two lines.
	 */

	h_line = ( end_io == 1 ) ? 1 : end_io + 2;

	mvaddstr( h_line,  3, "Pass" );
	mvaddstr( h_line, 11, "Thou" );
	mvaddstr( h_line, 20, "Exp" );
	mvaddstr( h_line, 28, "Sin" );
	mvaddstr( h_line, 36, "Cos" );
	mvaddstr( h_line, 44, "Tan" );
	mvaddstr( h_line, 52, "Squ" );
}

/********************************************************************
 *
 *	TITLE:	   genhead
 *
 *	CALLING SEQUENCE:
 *
 *		   genhead();
 *
 *	INTERFACE VARIABLES:
 *
 *		   .
 *              
 *	CALLS:	   .
 *
 *	ABSTRACT:  .
 *
 *
 *********************************************************************/
genhead()
{

	clear();
	standout();

	/*
	 *  Output headers to stdscr window
	 */

	if ( end_io > 1 ) {
		
		/*
		 *  Generate header and bars for io tests
		 */

		h_io();
		gen_bars( 0, end_io, TRUE );

	}

	if ( end_8087 > 1 ) {
		
		/*
		 *  Generate header and bars for 8087 test
		 */

		h_8087();

		if ( end_io == 1 )
			gen_bars( 0, end_8087, FALSE );
		else
			gen_bars( end_io + 2, end_8087, FALSE );
	}

	/*
	 *  Display Version number
	 */

	mvaddstr( 22, 0, "XSAT, Version 1.1" );

	/*
	 *  Display test time header
	 */

	mvaddstr( 21, 64, "TEST TIME" );
	
	if ( idleps ) {

		/*
		 *  Display idle time headers
		 */

		mvaddstr( 21, 48, "C-IDLE" );
		mvaddstr( 21, 55, "T-IDLE" );
		mvaddstr( 22, 53, "%" );
		mvaddstr( 22, 60, "%" );
	}

	/*
	 *  Refresh stdscr window to curscr window and screen.
	 */

	standend();
	refresh();
}
/********************************************************************
 *
 *	TITLE:	   repaint
 *
 *	CALLING SEQUENCE:
 *
 *		   repaint();
 *
 *	INTERFACE VARIABLES:
 *
 *		   .
 *              
 *	CALLS:	   .
 *
 *	ABSTRACT:  .
 *
 *
 *********************************************************************/
repaint()
{
struct msg_dsc msgs;	/*  Message structure for Curses filter	*/

	for ( ; ; ) {
	
		/*
		 *  Sleep for awhile and then refresh the headers.
		 */

		sleep( h_update );
	
		/*
		 *  Refresh curscr window to screen.
		 */

		refscr( &msgs );

	}
}

