/********************************************************************
 *
 *	TITLE:	   x8087
 *
 *	DATE:	   05-26-82
 *
 *	ABSTRACT:  .
 *
 *	LANGUAGE DEPENDENCIES:
 *
 *		C86     VERSION 1.0
 *
 *********************************************************************/
#include <math.h>
#include <curses.h>
#include "common.def" 

	/*
	 *  Definitions
	 */

#define max_real	1.0E+38
#define epsilon		1.0E-9
#define	delta		1.00025

	/*
	 *  External variables
	 */

extern int t_line;
extern int go[];
extern int synch[];

/********************************************************************
 *
 *	TITLE:	   normalize
 *
 *	CALLING SEQUENCE:
 *
 *		   result = normalize( actual, computed );
 *
 *	INTERFACE VARIABLES:
 *
 *		   .
 *              
 *	CALLS:	   .
 *
 *	ABSTRACT:  .
 *
 *
 *********************************************************************/
double normalize( actual, computed )
double actual, computed;
{
double a_exponent;
double c_exponent;
double norm;

	/*
	 *  Get exponents of acutal and computed values
	 */
	a_exponent = (int) log10( actual );
	c_exponent = (int) log10( computed );

	norm = pow( 10.0, ( a_exponent + c_exponent ) / 2.0 );

	return ( fabs( actual - computed ) / norm );
}
/********************************************************************
 *
 *	TITLE:	   x8087
 *
 *	CALLING SEQUENCE:
 *
 *		   if ( fork() == 0 )
 *			x8087();
 *
 *	INTERFACE VARIABLES:
 *
 *		   .
 *              
 *	CALLS:	   .
 *
 *	ABSTRACT:  .
 *
 *
 *********************************************************************/
x8087()
{
char c;
unsigned err_exp;
unsigned err_sin;
unsigned err_cos;
unsigned err_tan;
unsigned err_sqr;
unsigned i;
unsigned times;
double pi_over_2;
double x;
double xx;
double y;
double sign;
int line;
int pass;
struct msg_dsc msgs;


	pi_over_2 = atan( 1.0E+38 );

	line = t_line;

	i = write( synch[1], "done", 4 );
	if ( i == E_IO )
		err_msg( "Bad 8087 write of synch pipe","", TRUE  );

	i = read( go[0], &c, 1 );
	if ( i == E_IO )
		err_msg( "Bad 8087 read of go pipe", "", TRUE );

	/*
	 *  Initialize the 8087 test
	 */

	err_exp = 0;
	err_sin = 0;
	err_cos = 0;
	err_tan = 0;
	err_sqr = 0;
	pass = 0;

	sprintf( msgs.msg, "%5u", err_exp );
	xadd( &msgs, line, 18 );
	sprintf( msgs.msg, "%5u", err_sin );
	xadd( &msgs, line, 26 );
	sprintf( msgs.msg, "%5u", err_cos );
	xadd( &msgs, line, 34 );
	sprintf( msgs.msg, "%5u", err_tan );
	xadd( &msgs, line, 42 );
	sprintf( msgs.msg, "%5u", err_sqr );
	xadd( &msgs, line, 50 );
	xrefresh( &msgs );

	/*
	 *  Perform 8087 test
	 */

	for ( ; ; ) {


		/*
		 *  Display pass number
		 */

		++pass;
		times = 0;
		sign = 1.0;

		sprintf( msgs.msg, "%5u", pass );
		xadd( &msgs, line, 02 );
		sprintf( msgs.msg, "%5u", times );
		xadd( &msgs, line, 10 );
		xrefresh( &msgs );
		
		for ( i = 1, x = 1.0; x < max_real; i++ ) {

			y = exp( log( x ) );
			if ( normalize( x, y ) >= epsilon  ) {
				sprintf( msgs.msg, "%5u", ++err_exp );
				xadd( &msgs, line, 18 );
				xrefresh( &msgs );
			}

			/*
			 *  Compute a number between -1 and 1 for the 
			 *  sine and cosine tests.
			 */
			sign = -sign;
			xx = sign * x / pow( 10.0, (double) ((int) log10(x) + 1) );

			y = sin( asin( xx ) );
			if ( fabs( y - xx ) >= epsilon ) {
				sprintf( msgs.msg, "%5u", ++err_sin ); 
				xadd( &msgs, line, 26 ); 
				xrefresh( &msgs );
			}

			y = cos( acos( xx ) );
			if ( fabs( y - xx ) >= epsilon ) {
				sprintf( msgs.msg, "%5u", ++err_cos );
				xadd( &msgs, line, 34 );
				xrefresh( &msgs );
			}

			/*
			 *  Compute a number between -PI/2 and PI/2.
			 */
			xx = ( xx * 10.0 );
			xx = xx - ( pi_over_2 * (int) ( xx / pi_over_2 ) );

			y = atan( tan( xx ) );  
			if ( fabs( y - xx ) >= epsilon ) {
				sprintf( msgs.msg, "%5u", ++err_tan );
				xadd( &msgs, line, 42 );
				xrefresh( &msgs );
			}

			y = pow( sqrt( x ), 2.0 ); 
			if ( normalize( x, y ) >= epsilon ) {
				sprintf( msgs.msg, "%5u", ++err_sqr );
				xadd( &msgs, line, 50 );
				xrefresh( &msgs );
			}

			if ( i % 1000 == 0 ) {
				sprintf( msgs.msg, "%5u", ++times );
				xadd( &msgs, line, 10 );
				xrefresh( &msgs );
			}

			x *= delta;

		}
	}
}

